function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];
      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }
    return target;
  };
  return _extends.apply(this, arguments);
}

var omitMultiple = function omitMultiple(o, keys) {
  var exclude = new Set(keys);
  return Object.fromEntries(Object.entries(o).filter(function (e) {
    return !exclude.has(e[0]);
  }));
};
var getType = function getType(type, space) {
  return "" + (space ? space + "::" : '') + type;
};
var getFieldsObj = function getFieldsObj(fields) {
  return fields.reduce(function (acc, curr) {
    var _extends2;
    var currKey;
    var currValue;
    if (typeof curr === 'object') {
      currKey = curr.key;
      currValue = curr.value;
    } else if (typeof curr === 'string') {
      currKey = curr;
      currValue = null;
    }
    return _extends({}, acc, (_extends2 = {}, _extends2[currKey] = currValue, _extends2));
  }, {});
};

var REDUX_FIELD_NAME = 'fields';
var SIMPLE_REDUX_FIELDS__OPEN = 'SIMPLE_REDUX_FIELDS__OPEN';
var SIMPLE_REDUX_FIELDS__CLOSE = 'SIMPLE_REDUX_FIELDS__CLOSE';
var SIMPLE_REDUX_FIELDS__SET = 'SIMPLE_REDUX_FIELDS__SET';

var _simpleReduxFieldRedu;
var initialState = {};
var reducer = function reducer(state, action) {
  if (state === void 0) {
    state = initialState;
  }
  var payload = action.payload;
  var space = payload && payload.space ? payload.space : '';
  switch (action.type) {
    case getType(SIMPLE_REDUX_FIELDS__OPEN, space):
      return fieldsSet(state, payload);
    case getType(SIMPLE_REDUX_FIELDS__CLOSE, space):
      return fieldsClose(state, payload);
    case getType(SIMPLE_REDUX_FIELDS__SET, space):
      return fieldsSet(state, payload);
    default:
      return state;
  }
};
var simpleReduxFieldReducer = (_simpleReduxFieldRedu = {}, _simpleReduxFieldRedu[REDUX_FIELD_NAME] = reducer, _simpleReduxFieldRedu);
var fieldsSet = function fieldsSet(state, payload) {
  var fieldsObj = payload.fieldsObj;
  return Object.keys(fieldsObj).reduce(function (acc, curr) {
    var value = fieldsObj[curr];
    if (typeof value === 'object' && value !== null) {
      var _extends2;
      return _extends({}, acc, (_extends2 = {}, _extends2[curr] = _extends({}, state[curr], value), _extends2));
    } else {
      var _extends3;
      return _extends({}, acc, (_extends3 = {}, _extends3[curr] = value, _extends3));
    }
  }, state);
};
var fieldsClose = function fieldsClose(state, payload) {
  return omitMultiple(state, payload.keys);
};

var fieldsOpen = function fieldsOpen(fieldsObj, space) {
  return {
    type: getType(SIMPLE_REDUX_FIELDS__OPEN, space),
    payload: {
      space: space,
      fieldsObj: fieldsObj
    }
  };
};
var fieldsClose$1 = function fieldsClose(keys, space) {
  return {
    type: getType(SIMPLE_REDUX_FIELDS__CLOSE, space),
    payload: {
      space: space,
      keys: keys
    }
  };
};
var fieldsSet$1 = function fieldsSet(fieldsObj, space) {
  return {
    type: getType(SIMPLE_REDUX_FIELDS__SET, space),
    payload: {
      space: space,
      fieldsObj: fieldsObj
    }
  };
};

var fieldsOpen$1 = function fieldsOpen$1(fields, namespace) {
  if (namespace === void 0) {
    namespace = '';
  }
  return function (dispatch) {
    dispatch(fieldsOpen(getFieldsObj(fields), namespace));
  };
};
var fieldsClose$2 = function fieldsClose(keys, namespace) {
  if (namespace === void 0) {
    namespace = '';
  }
  return function (dispatch) {
    dispatch(fieldsClose$1(keys, namespace));
  };
};
var fieldsSet$2 = function fieldsSet(fields, namespace) {
  if (namespace === void 0) {
    namespace = '';
  }
  return function (dispatch) {
    dispatch(fieldsSet$1(getFieldsObj(fields), namespace));
  };
};
var fieldSet = function fieldSet(key, value, namespace) {
  if (namespace === void 0) {
    namespace = '';
  }
  return function (dispatch) {
    dispatch(fieldsSet$2([{
      key: key,
      value: value
    }], namespace));
  };
};

var $fields = function $fields(state) {
  return state[REDUX_FIELD_NAME];
};

exports.$fields = $fields;
exports.fieldSet = fieldSet;
exports.fieldsClose = fieldsClose$2;
exports.fieldsOpen = fieldsOpen$1;
exports.fieldsSet = fieldsSet$2;
exports.simpleReduxFieldReducer = simpleReduxFieldReducer;
//# sourceMappingURL=index.js.map
